/* Distributed under the terms of GPLv3 or later. See COPYING for details. */

/*! \file logging.h
 *
 * \brief Definition of the logging system.
 */
#ifndef INCLUDE_LOGGING_H_
#define INCLUDE_LOGGING_H_

//! \brief Debug level logging (maximum verbosity).
#define LOG_DEBG 0
//! \brief Standard information level logging (default).
#define LOG_INFO 1
//! \brief Warning level logging (almost quiet).
#define LOG_WARN 2
//! \brief Error level logging (silent, unless in pain).
#define LOG_ERRO 3

//! \brief Macro to stringize code lines
#define TOSTRING(ARG) string(#ARG)

/*! \brief Logger class.
 *
 * Loggers are objects used to track the execution of a code, reporting activities
 * such as function calls and parameter settings. They can be used to inform the
 * user about the execution of operations at runtime (e.g. by printing messages to
 * the terminal), as well as to record the execution history in appropriate log
 * files. The `Logger` class offers an implementation of logging system complying
 * with the requirements of the NP_TMcode project.
 */
class Logger {
 protected:
  //! \brief Pointer to error stream.
  FILE *err_output;
  //! \brief Pointer to logging stream.
  FILE *log_output;
  //! \brief Last logged message.
  std::string last_message;
  //! \brief Threshold of logging level.
  int log_threshold;
  //! \brief Number of identical message repetitions.
  long repetitions;

 public:
  /*! \brief Logger instance constructor.
   *
   * \param threshold: `int` Threshold of the messages to be included in log. Can
   * be `LOG_DEBG` (log everything), `LOG_INFO` (give detailed information),
   * `LOG_WARN` (log odd looking effects), or `LOG_ERRO` (print error messages,
   * `always active). The default behaviour is `LOG_WARN`.
   * \param logging_output: `FILE *` Pointer to an output file for common messages
   * (optional, default is `stdout`).
   * \param error_output: `FILE *` Pointer to an output file for error messages
   * (optional, default is `stderr`).
   */
  Logger(int threshold, FILE *logging_output=stdout, FILE *error_output=stderr);

  /*! \brief Print a message to the error output.
   *
   * \param message: `string` The message to be printed.
   */
  void err(std::string message);

  /*! \brief Print a summary of recurrent messages and clear the stack.
   *
   * \param level: `int` The priority level (default is `LOG_DEBG = 0`).
   */
  void flush(int level=LOG_DEBG);

  /*! \brief Print a message, depending on its logging level.
   *
   * \param message: `string` The message to be printed.
   * \param level: `int` The priority level (default is `LOG_INFO = 1`).
   */
  void log(std::string message, int level=LOG_INFO);
  
  /*! \brief Push a recurrent message to the message stack.
   *
   * When a long stream of identical messages is expected, it may be more
   * convenient to put them in a stack. The stack is the error output stream,
   * so that no memory is consumed. After the call stack is over, or is the
   * message changes, the stack is flushed, meaning that a summary message
   * is written to the logging output and the stack counter is reset to 0.
   *
   * \param message: `string` The message to be stacked.
   */
  void push(std::string message);
};

#endif
